searchData={"items":[{"type":"module","title":"BinStruct","doc":"","ref":"BinStruct.html"},{"type":"module","title":"Overview - BinStruct","doc":"```\n iex> defmodule SimpleChildStruct do\n ...>  use BinStruct\n ...>  field :data, :uint8\n ...> end\n ...>\n ...> defmodule SimpleStructWithChild do\n ...>   use BinStruct\n ...>   field :child, SimpleChildStruct\n ...> end\n ...>\n ...> SimpleStructWithChild.new(child: SimpleChildStruct.new(data: 1))\n ...> |> SimpleStructWithChild.dump_binary()\n ...> |> SimpleStructWithChild.parse()\n ...> |> then(fn {:ok, struct, _rest } -> struct end)\n ...> |> SimpleStructWithChild.decode()\n %{ child: SimpleChildStruct.new(data: 1) }\n\n```\n\nAs you can see from example on above parsed structs and newly created are always equal thanks to intermediate type conversion called `unmanaged`.\nIt's neither binary or managed and you are not suppose to work with it directly, by any type (including custom types) can perform\nautomatic type conversion between `binary`, `managed` and `unmanaged` on developer request (using `registered_callback` api)\n\nBinStruct will automatically generate set if functions for you:\n\n1. `dump_binary/1`\n2. `size/1`\n3. `parse/2` will be present if is terminated (have rules defined to be parsed into finite struct from infinity bytestream)\n4. `parse_exact/2`\n5. `decode/2`\n6. `new/1`\n\nIn additional with configuration it supports for now:\n\n```elixir\ntls_receive()\ntls_send()\ntcp_receive()\ntcp_send()\n```","ref":"BinStruct.html#module-overview"},{"type":"macro","title":"BinStruct.field/3","doc":"","ref":"BinStruct.html#field/3"},{"type":"macro","title":"Overview - BinStruct.field/3","doc":"With fields you are building the shape of your binary data.\n\n  field/3 expected you to pass name and type of your field. Supported types can be found in Se more detailed explanation in [`types` doc](pages/types/binary.md).\n  In additional you can pass another BinStruct itself and BinStructCustomType as type.","ref":"BinStruct.html#field/3-overview"},{"type":"macro","title":"Supported Options - BinStruct.field/3","doc":"","ref":"BinStruct.html#field/3-supported-options"},{"type":"macro","title":"length and its length_by dynamic version - BinStruct.field/3","doc":"```\n  field :value, :binary, length: 1\n  field :value, :binary, length_by: &callback/1\n  ```\n\n  length expect you to pass integer and field will be set strict to this length\n  same for length_by but it receiving callback returning integer instead","ref":"BinStruct.html#field/3-length-and-its-length_by-dynamic-version"},{"type":"macro","title":"validate_by - BinStruct.field/3","doc":"```\n  field :value, :uint8, validate_by: &callback/1\n  ```\n\n  Expecting callback returning true of data is valid and false if not\n  In case field is invalid parse will stop and { :wrong_data, _wrong_data_binary } will be returned.\n  Used by dynamic variant by dispatching.","ref":"BinStruct.html#field/3-validate_by"},{"type":"macro","title":"optional - BinStruct.field/3","doc":"```\n  field :v_always, :uint8\n  field :v_opt_tail1, :uint8, optional: true\n  field :v_opt_tail2, :uint8, optional: true\n  ```\n\n  Optional, also known as optional tail is the way stop parsing struct when\n  there is no more binary data and left all optional fields which not populated set to nil","ref":"BinStruct.html#field/3-optional"},{"type":"macro","title":"optional_by - BinStruct.field/3","doc":"```\n  field :value, :uint8, optional_by: &callback/1\n  ```\n\n  Conditionally present or not value.\n\n  Callback should return either true if value should be present or false otherwise.","ref":"BinStruct.html#field/3-optional_by"},{"type":"macro","title":"item_size and item_size_by (`list_of` only) - BinStruct.field/3","doc":"```\n  field :value, { :list_of, Item }, item_size: 2\n  field :value, { :list_of, Item }, item_size_by:  &callback/1\n  ```\n\n Se more detailed explanation in [`list_of` doc](pages/types/list_of.md)","ref":"BinStruct.html#field/3-item_size-and-item_size_by-list_of-only"},{"type":"macro","title":"count and count_by (`list_of` only) - BinStruct.field/3","doc":"```\n  field :value, { :list_of, Item }, count: 2\n  field :value, { :list_of, Item }, count_by: &callback/1\n ```\n\n Se more detailed explanation in [`list_of` doc](pages/types/list_of.md)","ref":"BinStruct.html#field/3-count-and-count_by-list_of-only"},{"type":"macro","title":"take_while_by (`list_of` only) - BinStruct.field/3","doc":"```\n  field :value, { :list_of, Item }, take_while_by: &callback/1\n  ```\n\n Se more detailed explanation in [`list_of` doc](pages/types/list_of.md)","ref":"BinStruct.html#field/3-take_while_by-list_of-only"},{"type":"macro","title":"builder - BinStruct.field/3","doc":"```\n  field :value, :uint8, builder: &callback/1\n  ```\n\n Builder callback will automatically build value for field.\n It should return 'managed' value, type conversion to underlying 'unmanaged' and 'binary' will be applied automatically.\n Builder callback called when you are creating struct via new/1 function.\n Builder can't request option argument, options are parse only tools.\n Field built with Builder can be requested from another builder, introducing chain. Calling order will be resolved automatically.","ref":"BinStruct.html#field/3-builder"},{"type":"macro","title":"BinStruct.impl_interface/2","doc":"Called in module with use BinStruct defined will implement options interface after struct fully parsed.\n\n```\n\n    iex> defmodule SharedOptions do\n    ...>   use BinStructOptionsInterface\n    ...>\n    ...>   @type shared_option :: :a | :b\n    ...>\n    ...>   register_option :shared_option\n    ...>\n    ...> end\n    ...>\n    ...>\n    ...>  defmodule StructImplementingOptionsInterface do\n    ...>   use BinStruct\n    ...>\n    ...>   register_callback &impl_options_interface_1/1, data: :field\n    ...>\n    ...>   impl_interface SharedOptions, &impl_options_interface_1/1\n    ...>\n    ...>   field :data, :binary, length: 1\n    ...>\n    ...>   defp impl_options_interface_1(\"A\"), do: SharedOptions.option_shared_option(:a)\n    ...>   defp impl_options_interface_1(\"B\"), do: SharedOptions.option_shared_option(:b)\n    ...>\n    ...> end\n    ...>\n    ...> defmodule ParentOfStructImplementingOptionsInterface do\n    ...>   use BinStruct\n    ...>\n    ...>   register_callback &dependent_field_len/1, shared_option: %{ type: :option, interface: SharedOptions }\n    ...>\n    ...>   field :child, StructImplementingOptionsInterface\n    ...>   field :dependent_field, :binary, length_by: &dependent_field_len/1\n    ...>\n    ...>   defp dependent_field_len(:a), do: 1\n    ...>   defp dependent_field_len(:b), do: 2\n    ...>\n    ...> end\n    ...>\n    ...> { :ok, _struct, \"\" = _rest } = ParentOfStructImplementingOptionsInterface.parse(\"A1\")\n    ...> { :ok, _struct, \"\" = _rest } = ParentOfStructImplementingOptionsInterface.parse(\"B22\")\n\n  ```","ref":"BinStruct.html#impl_interface/2"},{"type":"macro","title":"BinStruct.register_callback/2","doc":"Called in module with use BinStruct defined will register callback.\n\nRegisteredCallback is main source of dynamic behaviour.\n\nIt's binding arguments from current parse routine along with options from total parse tree context to function you pass in.\nAutomatically manipulating with type conversion on request.\n\n  ```\n\n    iex> defmodule StructWithRegisteredCallbackRequestField do\n    ...>   use BinStruct\n    ...>\n    ...>   register_callback &len_callback/1, len: :field\n    ...>\n    ...>   field :len, :uint32_be\n    ...>   field :data, :binary, length_by: &len_callback/1\n    ...>\n    ...>   defp len_callback(len), do: len\n    ...>\n    ...> end\n\n  ```\n\n  ```\n\n    iex> defmodule StructWithRegisteredCallbackRequestFieldInTypeConversion do\n    ...>   use BinStruct\n    ...>\n    ...>   alias BinStruct.TypeConversion.TypeConversionBinary\n    ...>\n    ...>   register_callback &payload_builder/1, number: %{ type: :field, type_conversion: TypeConversionBinary }\n    ...>\n    ...>   field :number, :uint32_be\n    ...>   field :payload, :binary, builder: &payload_builder/1\n    ...>\n    ...>   defp payload_builder(number_bin), do: number_bin\n    ...>\n    ...> end\n\n  ```\n\n  ```\n\n    iex> defmodule StructWithRegisteredCallbackRequestOption do\n    ...>   use BinStruct\n    ...>\n    ...>   register_option :opt\n    ...>\n    ...>   register_callback &data_length/1, opt: :option\n    ...>\n    ...>   field :data, :binary, length_by: &data_length/1\n    ...>\n    ...>   defp data_length(opt), do: opt\n    ...>\n    ...> end\n    ...>\n    ...> { :ok, struct, \"\" = _rest } = StructWithRegisteredCallbackRequestOption.parse(<<1>>, StructWithRegisteredCallbackRequestOption.option_opt(1))\n    ...> StructWithRegisteredCallbackRequestOption.decode(struct)\n    %{ data: <<1>> }\n\n  ```","ref":"BinStruct.html#register_callback/2"},{"type":"macro","title":"BinStruct.register_option/2","doc":"","ref":"BinStruct.html#register_option/2"},{"type":"macro","title":"Overview - BinStruct.register_option/2","doc":"Called in module with use BinStruct defined will register option with given name.\n\n  ```\n  register_option :name_of_opt\n  ```\n\n  It can be created using option_(your_option_name)(value) generated function\n\n  ```\n  YourBinStruct.option_name_of_opt(value)\n  ```\n\n  Multiple options are chained with pipe operator\n\n  ```\n  YourBinStruct.option_name_of_opt(value)\n  |> YourBinStruct.option_name_of_opt2(value)\n  |> YourBinStruct.option_name_of_opt3(value)\n  ```\n\n  If requested from same module it's registered in use short notation name_of_opt: :option\n\n  ```\n  register_callback &callback/1, name_of_opt:  :option\n  ```\n\n  Full notation is\n\n  ```\n  register_callback &callback/1, name_of_opt: { type: :option, interface: YourBinStruct }\n  ```","ref":"BinStruct.html#register_option/2-overview"},{"type":"macro","title":"BinStruct.virtual/3","doc":"","ref":"BinStruct.html#virtual/3"},{"type":"macro","title":"Overview - BinStruct.virtual/3","doc":"Virtual field is powerful system which can help you in many ways working with complex data.\n\nVirtual field will act as Field at most cases, including automatic type conversion.\n\nIt can be represented as any type available for Field and in special :unspecified form.\n\nSetting virtual field to :unspecified type will make automatic type conversions impossible but will give you opportunity\nto set it to any elixir term.","ref":"BinStruct.html#virtual/3-overview"},{"type":"macro","title":"How virtual fields behave in new context (when creating new struct) - BinStruct.virtual/3","doc":"When creating new struct with virtual fields you suppose to provide value for virtual field or attach builder callback\nif you need intermediate value as cache.\n\nAny builder callbacks can read that value in any type conversion and create for you actual binary data.","ref":"BinStruct.html#virtual/3-how-virtual-fields-behave-in-new-context-when-creating-new-struct"},{"type":"macro","title":"How virtual fields behave in parse context (when parsing) - BinStruct.virtual/3","doc":"Virtual fields can be requested by any registered callback in any type conversion (in case it's :unspecified only 'managed') available.\n\nOnly values for virtual fields requested will be produced and exactly once per field and type conversion pair.\n\nThis behaviour can make them useful is a cache while parsing.\n\nAny virtual field in recursive dependency of requested virtual field will be produced as well and only once rule will still work.\n\nIf A depends on B and B itself is requested in same type conversion A and B producing callbacks will be called once per such virtual field.\n\nMore idea of how and why you can use it currently you can find in test/virtual_field_system\n\nI will try to provide more detailed info on this topic later.","ref":"BinStruct.html#virtual/3-how-virtual-fields-behave-in-parse-context-when-parsing"},{"type":"macro","title":"How virtual fields behave in decode context (when reading data) - BinStruct.virtual/3","doc":"They will be present in decoded data.","ref":"BinStruct.html#virtual/3-how-virtual-fields-behave-in-decode-context-when-reading-data"},{"type":"macro","title":"Supported Options - BinStruct.virtual/3","doc":"","ref":"BinStruct.html#virtual/3-supported-options"},{"type":"macro","title":"read_by - BinStruct.virtual/3","doc":"```\n  virtual :v_field_name, :uint8, read_by: &callback/1\n  ```\n\n  Source of this virtual field.\n\n  This callback will be called once.\n\n  On decode it's always called to produce output.\n\n  On parse this callback will be called only if this virtual field is direct or indirect dependency of other callbacks.","ref":"BinStruct.html#virtual/3-read_by"},{"type":"macro","title":"builder - BinStruct.virtual/3","doc":"```\nvirtual :v_field_name, :binary_utf16, builder: &callback/1\n```\n\nCallback which virtual field will be created from during creation of new struct.\n\nThis value could be used from any other builders with any requested automatic type conversion.\n\nVirtual field will act as intermediate storage while you creating new struct. Useful separation effect and cache effect.","ref":"BinStruct.html#virtual/3-builder"},{"type":"macro","title":"optional - BinStruct.virtual/3","doc":"```\n  virtual :v_field_name, :uint8, optional: true\n  ```\n  Makes field optional","ref":"BinStruct.html#virtual/3-optional"},{"type":"module","title":"BinStruct.BuiltIn.Recursive","doc":"A custom type for handling recursive structures within `BinStruct`.","ref":"BinStruct.BuiltIn.Recursive.html"},{"type":"module","title":"Overview - BinStruct.BuiltIn.Recursive","doc":"The `Recursive` custom type is designed to facilitate parsing, size computation, and binary serialization\nof nested or recursive structures in a way that allows the library to manage recursion efficiently.\n\nInstead of relying on the library to directly introspect recursive types during compile-time analysis,\nthis module enables wrapping a recursive type with a custom definition. By doing so, it delegates\nthe responsibility of compile-time introspection to the custom type itself. This allows for more\nprecise control over how the recursive type is handled and provides an avenue for customization\nin the future (though such customization is not yet implemented).","ref":"BinStruct.BuiltIn.Recursive.html#module-overview"},{"type":"module","title":"Example Usage - BinStruct.BuiltIn.Recursive","doc":"Instead of declaring a recursive type directly:\n\n```elixir\nfield :sequence, Sequence\n```\n\nWrap the recursive type with Recursive:\n\n```elixir\nfield :sequence, {Recursive, module: Sequence}\n```","ref":"BinStruct.BuiltIn.Recursive.html#module-example-usage"},{"type":"function","title":"BinStruct.BuiltIn.Recursive.dump_binary/2","doc":"","ref":"BinStruct.BuiltIn.Recursive.html#dump_binary/2"},{"type":"function","title":"BinStruct.BuiltIn.Recursive.from_managed_to_unmanaged/2","doc":"","ref":"BinStruct.BuiltIn.Recursive.html#from_managed_to_unmanaged/2"},{"type":"function","title":"BinStruct.BuiltIn.Recursive.from_unmanaged_to_managed/2","doc":"","ref":"BinStruct.BuiltIn.Recursive.html#from_unmanaged_to_managed/2"},{"type":"function","title":"BinStruct.BuiltIn.Recursive.init_args/1","doc":"","ref":"BinStruct.BuiltIn.Recursive.html#init_args/1"},{"type":"function","title":"BinStruct.BuiltIn.Recursive.known_total_size_bytes/1","doc":"","ref":"BinStruct.BuiltIn.Recursive.html#known_total_size_bytes/1"},{"type":"function","title":"BinStruct.BuiltIn.Recursive.module_code/0","doc":"","ref":"BinStruct.BuiltIn.Recursive.html#module_code/0"},{"type":"function","title":"BinStruct.BuiltIn.Recursive.parse_exact_returning_options/3","doc":"","ref":"BinStruct.BuiltIn.Recursive.html#parse_exact_returning_options/3"},{"type":"function","title":"BinStruct.BuiltIn.Recursive.parse_returning_options/3","doc":"","ref":"BinStruct.BuiltIn.Recursive.html#parse_returning_options/3"},{"type":"function","title":"BinStruct.BuiltIn.Recursive.size/2","doc":"","ref":"BinStruct.BuiltIn.Recursive.html#size/2"},{"type":"module","title":"BinStruct.BuiltIn.Asn1","doc":"Asn1 is `BinStructCustomType` and it uses erlang [`asn1`](https://www.erlang.org/doc/apps/asn1/api-reference.html) compiler output as implementation.\n\nAsn1 modules and types can be compiled using this command\n\n```bash\nerlc -o \"$OUTPUT_DIR\" -I \"$OUTPUT_DIR\" +noobj -bper +maps +undec_rest TEST-ASN1.asn1\n```\n\n`+maps` is optional, change `-bper` to your target encoding (erlang compiler support BER, PER and so on)\n`+undec_rest` is required to current `BinStructCustomType` implementation,\nbut this may be non-ideal and sometimes shadow errors in data, returning `:not_enough_bytes` instead\n\n\n```elixir\ndefmodule Struct do\n  use BinStruct\n\n  alias BinStruct.BuiltIn.Asn1\n\n  field :asn1, { Asn1, asn1_module: :\"TEST-ASN1\", asn1_type: :\"SimpleType\" }\nend\n```","ref":"BinStruct.BuiltIn.Asn1.html"},{"type":"function","title":"BinStruct.BuiltIn.Asn1.dump_binary/2","doc":"","ref":"BinStruct.BuiltIn.Asn1.html#dump_binary/2"},{"type":"function","title":"BinStruct.BuiltIn.Asn1.from_managed_to_unmanaged/2","doc":"","ref":"BinStruct.BuiltIn.Asn1.html#from_managed_to_unmanaged/2"},{"type":"function","title":"BinStruct.BuiltIn.Asn1.from_unmanaged_to_managed/2","doc":"","ref":"BinStruct.BuiltIn.Asn1.html#from_unmanaged_to_managed/2"},{"type":"function","title":"BinStruct.BuiltIn.Asn1.init_args/1","doc":"","ref":"BinStruct.BuiltIn.Asn1.html#init_args/1"},{"type":"function","title":"BinStruct.BuiltIn.Asn1.known_total_size_bytes/1","doc":"","ref":"BinStruct.BuiltIn.Asn1.html#known_total_size_bytes/1"},{"type":"function","title":"BinStruct.BuiltIn.Asn1.module_code/0","doc":"","ref":"BinStruct.BuiltIn.Asn1.html#module_code/0"},{"type":"function","title":"BinStruct.BuiltIn.Asn1.parse_exact_returning_options/3","doc":"","ref":"BinStruct.BuiltIn.Asn1.html#parse_exact_returning_options/3"},{"type":"function","title":"BinStruct.BuiltIn.Asn1.parse_returning_options/3","doc":"","ref":"BinStruct.BuiltIn.Asn1.html#parse_returning_options/3"},{"type":"function","title":"BinStruct.BuiltIn.Asn1.size/2","doc":"","ref":"BinStruct.BuiltIn.Asn1.html#size/2"},{"type":"module","title":"BinStruct.BuiltIn.TerminatedBinary","doc":"Terminated binary implemented as `BinStructCustomType`.\n\n```elixir\ndefmodule Struct do\n  use BinStruct\n\n  alias BinStruct.BuiltIn.TerminatedBinary\n\n  field :terminated_binary, { TerminatedBinary, termination: <<0, 0>> }\nend\n```","ref":"BinStruct.BuiltIn.TerminatedBinary.html"},{"type":"function","title":"BinStruct.BuiltIn.TerminatedBinary.dump_binary/2","doc":"","ref":"BinStruct.BuiltIn.TerminatedBinary.html#dump_binary/2"},{"type":"function","title":"BinStruct.BuiltIn.TerminatedBinary.from_managed_to_unmanaged/2","doc":"","ref":"BinStruct.BuiltIn.TerminatedBinary.html#from_managed_to_unmanaged/2"},{"type":"function","title":"BinStruct.BuiltIn.TerminatedBinary.from_unmanaged_to_managed/2","doc":"","ref":"BinStruct.BuiltIn.TerminatedBinary.html#from_unmanaged_to_managed/2"},{"type":"function","title":"BinStruct.BuiltIn.TerminatedBinary.init_args/1","doc":"","ref":"BinStruct.BuiltIn.TerminatedBinary.html#init_args/1"},{"type":"function","title":"BinStruct.BuiltIn.TerminatedBinary.known_total_size_bytes/1","doc":"","ref":"BinStruct.BuiltIn.TerminatedBinary.html#known_total_size_bytes/1"},{"type":"function","title":"BinStruct.BuiltIn.TerminatedBinary.module_code/0","doc":"","ref":"BinStruct.BuiltIn.TerminatedBinary.html#module_code/0"},{"type":"function","title":"BinStruct.BuiltIn.TerminatedBinary.parse_exact_returning_options/3","doc":"","ref":"BinStruct.BuiltIn.TerminatedBinary.html#parse_exact_returning_options/3"},{"type":"function","title":"BinStruct.BuiltIn.TerminatedBinary.parse_returning_options/3","doc":"","ref":"BinStruct.BuiltIn.TerminatedBinary.html#parse_returning_options/3"},{"type":"function","title":"BinStruct.BuiltIn.TerminatedBinary.size/2","doc":"","ref":"BinStruct.BuiltIn.TerminatedBinary.html#size/2"},{"type":"module","title":"BinStruct.EnumValueByVariantName","doc":"Useful in `registered_callbacks` when library can't achieve automatic type conversion.\n\n```elixir\niex> enum = [{0x01, :a}, {0x02, :b}]\n...> BinStruct.EnumValueByVariantName.find_enum_value_by_variant_name(enum, :a)\n0x01\n```","ref":"BinStruct.EnumValueByVariantName.html"},{"type":"function","title":"BinStruct.EnumValueByVariantName.find_enum_value_by_variant_name/2","doc":"","ref":"BinStruct.EnumValueByVariantName.html#find_enum_value_by_variant_name/2"},{"type":"module","title":"BinStruct.EnumVariantNameByValue","doc":"Useful in `registered_callbacks` when library can't achieve automatic type conversion.\n\n```elixir\niex> enum = [{ 0x01, :a}, {0x02, :b}]\n...> BinStruct.EnumVariantNameByValue.find_enum_variant_name_by_value(enum, 0x01)\n:a\n```","ref":"BinStruct.EnumVariantNameByValue.html"},{"type":"function","title":"BinStruct.EnumVariantNameByValue.find_enum_variant_name_by_value/2","doc":"","ref":"BinStruct.EnumVariantNameByValue.html#find_enum_variant_name_by_value/2"},{"type":"module","title":"BinStruct.FlagsReader","doc":"Useful in `registered_callbacks` when library can't achieve automatic type conversion.\n\n```elixir\niex> flags = [{0x01, :flag_a}, {0x02, :flag_b}]\n...> BinStruct.FlagsReader.read_flags_from_integer(flags, 0x01)\n[:flag_a]\n```","ref":"BinStruct.FlagsReader.html"},{"type":"function","title":"BinStruct.FlagsReader.read_flags_from_integer/2","doc":"","ref":"BinStruct.FlagsReader.html#read_flags_from_integer/2"},{"type":"module","title":"BinStruct.FlagsWriter","doc":"Useful in `registered_callbacks` when library can't achieve automatic type conversion.\n\n```elixir\niex> flags = [{ 0x01, :flag_a}, {0x02, :flag_b}]\n...> BinStruct.FlagsWriter.write_flags_to_integer(flags, [:flag_a])\n0x01\n```","ref":"BinStruct.FlagsWriter.html"},{"type":"function","title":"BinStruct.FlagsWriter.write_flags_to_integer/2","doc":"","ref":"BinStruct.FlagsWriter.html#write_flags_to_integer/2"},{"type":"module","title":"BinStruct.PrimitiveEncoder","doc":"Useful in `registered_callbacks` when library can't achieve automatic type conversion.\n\n```elixir\niex> BinStruct.PrimitiveEncoder.uint8(0)\n<<0>>\n```","ref":"BinStruct.PrimitiveEncoder.html"},{"type":"function","title":"BinStruct.PrimitiveEncoder.bool/2","doc":"","ref":"BinStruct.PrimitiveEncoder.html#bool/2"},{"type":"function","title":"BinStruct.PrimitiveEncoder.float32_be/1","doc":"","ref":"BinStruct.PrimitiveEncoder.html#float32_be/1"},{"type":"function","title":"BinStruct.PrimitiveEncoder.float32_le/1","doc":"","ref":"BinStruct.PrimitiveEncoder.html#float32_le/1"},{"type":"function","title":"BinStruct.PrimitiveEncoder.float64_be/1","doc":"","ref":"BinStruct.PrimitiveEncoder.html#float64_be/1"},{"type":"function","title":"BinStruct.PrimitiveEncoder.float64_le/1","doc":"","ref":"BinStruct.PrimitiveEncoder.html#float64_le/1"},{"type":"function","title":"BinStruct.PrimitiveEncoder.int8/1","doc":"","ref":"BinStruct.PrimitiveEncoder.html#int8/1"},{"type":"function","title":"BinStruct.PrimitiveEncoder.int16_be/1","doc":"","ref":"BinStruct.PrimitiveEncoder.html#int16_be/1"},{"type":"function","title":"BinStruct.PrimitiveEncoder.int16_le/1","doc":"","ref":"BinStruct.PrimitiveEncoder.html#int16_le/1"},{"type":"function","title":"BinStruct.PrimitiveEncoder.int32_be/1","doc":"","ref":"BinStruct.PrimitiveEncoder.html#int32_be/1"},{"type":"function","title":"BinStruct.PrimitiveEncoder.int32_le/1","doc":"","ref":"BinStruct.PrimitiveEncoder.html#int32_le/1"},{"type":"function","title":"BinStruct.PrimitiveEncoder.int64_be/1","doc":"","ref":"BinStruct.PrimitiveEncoder.html#int64_be/1"},{"type":"function","title":"BinStruct.PrimitiveEncoder.int64_le/1","doc":"","ref":"BinStruct.PrimitiveEncoder.html#int64_le/1"},{"type":"function","title":"BinStruct.PrimitiveEncoder.int_be/2","doc":"","ref":"BinStruct.PrimitiveEncoder.html#int_be/2"},{"type":"function","title":"BinStruct.PrimitiveEncoder.int_le/2","doc":"","ref":"BinStruct.PrimitiveEncoder.html#int_le/2"},{"type":"function","title":"BinStruct.PrimitiveEncoder.uint8/1","doc":"","ref":"BinStruct.PrimitiveEncoder.html#uint8/1"},{"type":"function","title":"BinStruct.PrimitiveEncoder.uint16_be/1","doc":"","ref":"BinStruct.PrimitiveEncoder.html#uint16_be/1"},{"type":"function","title":"BinStruct.PrimitiveEncoder.uint16_le/1","doc":"","ref":"BinStruct.PrimitiveEncoder.html#uint16_le/1"},{"type":"function","title":"BinStruct.PrimitiveEncoder.uint32_be/1","doc":"","ref":"BinStruct.PrimitiveEncoder.html#uint32_be/1"},{"type":"function","title":"BinStruct.PrimitiveEncoder.uint32_le/1","doc":"","ref":"BinStruct.PrimitiveEncoder.html#uint32_le/1"},{"type":"function","title":"BinStruct.PrimitiveEncoder.uint64_be/1","doc":"","ref":"BinStruct.PrimitiveEncoder.html#uint64_be/1"},{"type":"function","title":"BinStruct.PrimitiveEncoder.uint64_le/1","doc":"","ref":"BinStruct.PrimitiveEncoder.html#uint64_le/1"},{"type":"function","title":"BinStruct.PrimitiveEncoder.uint_be/2","doc":"","ref":"BinStruct.PrimitiveEncoder.html#uint_be/2"},{"type":"function","title":"BinStruct.PrimitiveEncoder.uint_le/2","doc":"","ref":"BinStruct.PrimitiveEncoder.html#uint_le/2"},{"type":"module","title":"BinStructCustomType","doc":"`BinStructCustomType` is user defined type when you need most control of how data are parsed.\n\nSuch custom type can be used in place you would use BinStruct.\n\nThe only currently limitation it can't be used directly inside variant_of (variant_of dispatching relays on elixir struct).\n\nRequired functions to be defined by custom type:\n\n* either `parse_returning_options/3` or `parse_exact_returning_options/3`\n* `size/2`\n* `known_total_size_bytes/2`\n* `dump_binary/2`\n* `from_unmanaged_to_managed/2`\n* `from_managed_to_unmanaged/2`\n\nOptional functions to be defined by custom type:\n\n* `init_args/1`","ref":"BinStructCustomType.html"},{"type":"module","title":"Basic Example - BinStructCustomType","doc":"```elixir\ndefmodule SimpleCustomTypeTwoBytesLong do\n  use BinStructCustomType\n\n  def parse_returning_options(bin, _custom_type_args, opts) do\n    case bin do\n      < > -> {:ok, data, rest, opts}\n      _ -> :not_enough_bytes\n    end\n  end\n\n  def size(_data, _custom_type_args), do: 2\n  def known_total_size_bytes(_custom_type_args), do: 2\n  def dump_binary(data, _custom_type_args), do: data\n  def from_unmanaged_to_managed(unmanaged, _custom_type_args), do: unmanaged\n  def from_managed_to_unmanaged(managed, _custom_type_args), do: managed\nend\n```","ref":"BinStructCustomType.html#module-basic-example"},{"type":"macro","title":"BinStructCustomType.register_option/2","doc":"Registering option for custom type. Interface will be your module name.\nImplementation and usage same as register_option for BinStruct.\n\nSee more in `BinStruct.register_option/2`","ref":"BinStructCustomType.html#register_option/2"},{"type":"module","title":"BinStructOptionsInterface","doc":"```elixir\niex> defmodule SharedOptions do\n...>   use BinStructOptionsInterface\n...>\n...>   @type runtime_context1 :: :context_a\n...>   @type runtime_context2 :: :context_b\n...>\n...>   register_option :runtime_context1\n...>   register_option :runtime_context2\n...>\n...> end\n...>\n...> SharedOptions.option_runtime_context1(:context_a)\n...> |> SharedOptions.option_runtime_context2(:context_b)\n```","ref":"BinStructOptionsInterface.html"},{"type":"macro","title":"BinStructOptionsInterface.register_option/2","doc":"Registering option for option interface. Interface will be your module name.\nImplementation and usage same as `register_option` for `BinStruct`.\n\nSee more in `BinStruct.register_option/2`","ref":"BinStructOptionsInterface.html#register_option/2"},{"type":"extras","title":"BinStruct","doc":"# BinStruct\n\n**BinStruct** is a library that provides a rich set of tools for parsing and encoding binaries.\n\nThe goal is to write declarations that are **readable now and will remain readable for years**. Your code will closely resemble a page from protocol documentation, ensuring clarity and maintainability.\n\nThis library is particularly beneficial for use cases that require **bidirectional data flow**.","ref":"readme.html"},{"type":"extras","title":"What BinStruct is not - BinStruct","doc":"BinStruct is not an protocol itself, there is no goal to replace asn1, protobuf, erlang binary term or any other protocols. if you can solve your problem using existing protocol - stick with it.\n\nBinStruct is not replacement for binary pattern matching. If your job can be done via pattern match only it will be always better to use it directly.\nThere is layer of complexity this lib adds to make it achive it's main goal - write declarations, generate implementations automatically. When complexity grows only sane way to keep with it has general declarative structure of each part you working this.\n\nBinStruct is by no means a framework and does not force you to follow any specific structure of how its parts will be used together. \nEach BinStruct you create is completely self-contained and can be used as you see fit. Whether you want to validate CRC, add encryption, or implement something else inside or outside—it’s entirely up to you, and the library imposes no restrictions on these choices.","ref":"readme.html#what-binstruct-is-not"},{"type":"extras","title":"What BinStruct is primarily - BinStruct","doc":"BinStruct is s tool. Tool to support developer from very beggining with reach set of generated features, allowing to exlore data in every step, to very end running your app in production.\n\nI believe BinStruct is an essential tool for developers. Simply transferring declarations from your protocol documentation into BinStruct special syntax is enough to start parsing your data, decoding it, and exploring its structure. This lets you build an understanding of how to proceed next.\nIt is especially helpful when working with a protocol that is new to you. If you’re unsure where to start or what to focus on, just transfer what you see in the documentation into BinStruct declarations and experiment. At some point, things will start falling into place, and you might even find that the application almost writes itself before you realize it.\nEven the smallest fragments you implement can already be put to use. You can parse and decode binary data to gain a better understanding of what you're dealing with without needing to fully implement every detail or dynamic callback. You'll gradually build out your protocol implementation step by step, and over time, these pieces will naturally connect as your codebase grows.\nYou don’t need all the advanced features like virtual fields, auto-generated fields (builders), or type conversions beyond the basic managed (human-readable) one right away. You can always add them later if you think they’ll make the process easier.","ref":"readme.html#what-binstruct-is-primarily"},{"type":"extras","title":"Installation - BinStruct","doc":"To get started, add `BinStruct` to your `mix.exs` dependencies:\n\n```elixir\ndef deps do\n  [\n    {:bin_struct, \"~> 0.2\"}\n  ]\nend\n```\n\n---","ref":"readme.html#installation"},{"type":"extras","title":"Syntax overview - BinStruct","doc":"```elixir\n  \n  defmodule PngChunk do\n  \n    use BinStruct\n  \n    #all dynamic behaviour is callback\n    #if we are not specifying type_conversion this is always 'managed' also known as 'human readable'\n    register_callback &data_length/1, length: :field\n  \n    #with fields you build shape of your binary data\n    field :length, :uint32_be\n  \n    #use expanded constructs whenever possible, this is both easier to read and will be validated at parse time\n    #its always better to expand arrays/flags/enums even if you don't use them for now, it will help moving forward\n    #as you will have more complete picture\n    #and also it will give you opportunity to be dispatched as dynamic variant later (read it as if we received something and it has type distinct from listed below it's not this struct, we can catch it via upper variant_of later)\n    field :type, {\n      :enum,\n      %{\n        type: :binary,\n        values: [\n          \"IHDR\",\n          \"PLTE\",\n          \"IDAT\",\n          \"IEND\",\n          \"cHRM\",\n          \"gAMA\",\n          \"iCCP\",\n          \"sBIT\",\n          \"sRGB\",\n          \"bKGD\",\n          \"hIST\",\n          \"tRNS\",\n          \"pHYs\",\n          \"sPLT\",\n          \"tIME\",\n          \"tEXt\",\n          \"zTXt\",\n          \"iTXt\"\n        ]\n      }\n    }, length: 4\n  \n    #consuming dynamic behaviour into length_by\n    field :data, :binary, length_by: &data_length/1\n  \n    field :crc, :uint32_be\n  \n    #dynamic behaviour implementation\n    #we returning always 'managed' type conversion, in this case length field will be automatically converted to elixir number\n    #and we return this number as it\n    defp data_length(length), do: length\n  \n  end\n  \n  ```","ref":"readme.html#syntax-overview"},{"type":"extras","title":"Getting Started - BinStruct","doc":"","ref":"readme.html#getting-started"},{"type":"extras","title":"Explore Examples - BinStruct","doc":"Start by exploring the examples folder. Run the following commands to see an example in action:\n\n```sh\nmix run examples/png.exs\nmix run examples/extraction_from_integer.exs\nmix run examples/extraction_from_buffer.exs\nmix run examples/packet_via_higher_order_macro.exs\nmix run examples/recursive_sequence.exs\n```","ref":"readme.html#explore-examples"},{"type":"extras","title":"Reference Documentation - BinStruct","doc":"- [BinStruct Module Documentation](https://hexdocs.pm/bin_struct/BinStruct.html)\n\n#### Type-Specific Docs\n\n- [StaticValue](https://hexdocs.pm/bin_struct/static_value.html)\n- [Binary](https://hexdocs.pm/bin_struct/binary.html)\n- [Enum](https://hexdocs.pm/bin_struct/enum.html)\n- [Flags](https://hexdocs.pm/bin_struct/flags.html)\n- [List](https://hexdocs.pm/bin_struct/list_of.html)\n- [Variant](https://hexdocs.pm/bin_struct/variant_of.html)","ref":"readme.html#reference-documentation"},{"type":"extras","title":"Explore Tests - BinStruct","doc":"For further insights, check out the `test/` folder, where you can explore the library’s future functionality through tests.","ref":"readme.html#explore-tests"},{"type":"extras","title":"View generated code - BinStruct","doc":"there is module_code/0 function for any module with applied use macro to (BinStruct, BinStructCustomType or BinStructOptionsInterface)\nwhich will show for you all generated code.","ref":"readme.html#view-generated-code"},{"type":"extras","title":"Configuration - BinStruct","doc":"You can configure `BinStruct` by adding the following to your `config.exs` file:\n\n```elixir\nconfig :bin_struct,\n  define_receive_send_tcp: false, #default false\n  define_receive_send_tls: false, #default false\n  enable_log_tcp: true, #default true\n  enable_log_tls: true #default true\n```","ref":"readme.html#configuration"},{"type":"extras","title":"Notes on TLS - BinStruct","doc":"- TLS functionality is implemented using the `:ssl` application.\n- If you wish to use TLS, make sure to either **disable it** or add `:ssl` to the list of `extra_applications`:\n\n  ```elixir\n  def application do\n    [\n      extra_applications: [:ssl]\n    ]\n  end\n  ```","ref":"readme.html#notes-on-tls"},{"type":"extras","title":"Additional Documentation - BinStruct","doc":"Comprehensive documentation is available online:  \n[BinStruct Docs](https://hexdocs.pm/bin_struct)\n\n---","ref":"readme.html#additional-documentation"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"0.2.9 - Changelog","doc":"Changelog starts. Public release","ref":"changelog.html#0-2-9"},{"type":"extras","title":"0.2.10 - Changelog","doc":"Improvement to documentation","ref":"changelog.html#0-2-10"},{"type":"extras","title":"0.2.11 - Changelog","doc":"Another sequence of documentation updates","ref":"changelog.html#0-2-11"},{"type":"extras","title":"0.2.12 - Changelog","doc":"Added Recursive custom type for dynamic parsing recursive datastructures","ref":"changelog.html#0-2-12"},{"type":"extras","title":"asn1","doc":"# asn1\n\nSee `BinStruct.BuiltIn.Asn1` for more info","ref":"asn1.html"},{"type":"extras","title":"binary","doc":"# binary\n\nFor various types of byte sequences including non-terminated, fixed length or dynamic length.","ref":"binary.html"},{"type":"extras","title":"Terminated - binary","doc":"See `BinStruct.BuiltIn.TerminatedBinary`","ref":"binary.html#terminated"},{"type":"extras","title":"Known length - binary","doc":"```elixir\niex> defmodule StructKnownLength do\n...>   use BinStruct\n...>   field :value, :binary, length: 3\n...> end\n...>\n...> StructKnownLength.new(value: \"123\")\n...> |> StructKnownLength.dump_binary()\n...> |> StructKnownLength.parse()\n...> |> then(fn {:ok, struct, _rest} -> struct end)\n...> |> StructKnownLength.decode()\n%{value: \"123\"}\n```\n\n##  Dynamic length\n\n\n```elixir\niex> defmodule StructWithDynamicLength do\n...>   use BinStruct\n...>\n...>   register_callback &len/0\n...>   field :value, :binary, length_by: &len/0\n...>\n...>   defp len(), do: 3\n...> end\n...>\n...> StructWithDynamicLength.new(value: \"123\")\n...> |> StructWithDynamicLength.dump_binary()\n...> |> StructWithDynamicLength.parse()\n...> |> then(fn {:ok, struct, _rest} -> struct end)\n...> |> StructWithDynamicLength.decode()\n%{value: \"123\"}\n```","ref":"binary.html#known-length"},{"type":"extras","title":"Unknown length - binary","doc":"```elixir\niex> defmodule StructNonTerminated do\n...>   use BinStruct\n...>   field :value, :binary\n...> end\n...>\n...> StructNonTerminated.new(value: \"123\")\n...> |> StructNonTerminated.dump_binary()\n...> |> StructNonTerminated.parse_exact()\n...> |> then(fn {:ok, struct} -> struct end)\n...> |> StructNonTerminated.decode()\n%{value: \"123\"}\n```","ref":"binary.html#unknown-length"},{"type":"extras","title":"bool","doc":"# bool\n\nFor boolean fields of various bit sizes","ref":"bool.html"},{"type":"extras","title":"1 byte - bool","doc":"```elixir\n\n  iex> defmodule StructOneByteBool do\n  ...>   use BinStruct\n  ...>   field :value, :bool\n  ...> end\n  ...>\n  ...> StructOneByteBool.new(value: true)\n  ...> |> StructOneByteBool.dump_binary()\n  ...> |> StructOneByteBool.parse()\n  ...> |> then(fn {:ok, struct, _rest } -> struct end)\n  ...> |> StructOneByteBool.decode()\n  %{ value: true }\n\n```\n\n##  Variable bit size\n\n```elixir\n\n  iex> defmodule StructVariableBitSize do\n  ...>   use BinStruct\n  ...>   field :a1, :bool, bits: 1\n  ...>   field :a2, :bool, bits: 1\n  ...>   field :a3, :bool, bits: 1\n  ...>   field :a4, :bool, bits: 1\n  ...>   field :a5, :bool, bits: 1\n  ...>   field :a6, :bool, bits: 1\n  ...>   field :a7, :bool, bits: 1\n  ...>   field :a8, :bool, bits: 1\n  ...> end\n  ...>\n  ...> StructVariableBitSize.new(a1: true, a2: true, a3: true, a4: true, a5: true, a6: true, a7: true, a8: true)\n  ...> |> StructVariableBitSize.dump_binary()\n  ...> |> StructVariableBitSize.parse()\n  ...> |> then(fn {:ok, struct, _rest } -> struct end)\n  ...> |> StructVariableBitSize.decode()\n  %{ a1: true, a2: true, a3: true, a4: true, a5: true, a6: true, a7: true, a8: true }\n\n```","ref":"bool.html#1-byte"},{"type":"extras","title":"enum","doc":"# enum\n\nEnum can be declared as any internal type.\n\nIf none of value are matched while parsing { :wrong_data, _wrong_data_bin } will be returned for such struct.\nThis is playing nicely with dynamic variant dispatching, same as you would expect for static_value.\n\n\nHere’s an example:","ref":"enum.html"},{"type":"extras","title":"Integer based - enum","doc":"```elixir\n\n  iex> defmodule EnumInteger do\n  ...>   use BinStruct\n  ...>   field :value, {\n  ...>    :enum,\n  ...>    %{\n  ...>     type: :uint16_le,\n  ...>     values: [\n  ...>      {0x0004, :high_color_4bpp},\n  ...>      {0x0008, :high_color_8bpp}\n  ...>     ]\n  ...>    }\n  ...>  }\n  ...> end\n  ...>\n  ...> EnumInteger.new(value: :high_color_4bpp)\n  ...> |> EnumInteger.dump_binary()\n  ...> |> EnumInteger.parse()\n  ...> |> then(fn {:ok, struct, _rest } -> struct end)\n  ...> |> EnumInteger.decode()\n  %{ value: :high_color_4bpp }\n\n```","ref":"enum.html#integer-based"},{"type":"extras","title":"Binaries based - enum","doc":"```elixir\n\n  iex> defmodule EnumBinaries do\n  ...>   use BinStruct\n  ...>   field :value, {\n  ...>    :enum,\n  ...>    %{\n  ...>     type: :binary,\n  ...>     values: [\n  ...>      { \"A\" , :high_color_4bpp},\n  ...>      { \"B\", :high_color_8bpp}\n  ...>     ]\n  ...>    }\n  ...>  }, length: 1\n  ...> end\n  ...>\n  ...> EnumBinaries.new(value: :high_color_4bpp)\n  ...> |> EnumBinaries.dump_binary()\n  ...> |> EnumBinaries.parse()\n  ...> |> then(fn {:ok, struct, _rest } -> struct end)\n  ...> |> EnumBinaries.decode()\n  %{ value: :high_color_4bpp }\n\n```","ref":"enum.html#binaries-based"},{"type":"extras","title":"flags","doc":"# flags","ref":"flags.html"},{"type":"extras","title":"Defining struct with flags using integer mask - flags","doc":"Current implementation support flags extraction up to 64 bits long integer\n\n```elixir\n\n  iex> defmodule IntegerMaskFlags do\n  ...>   use BinStruct\n  ...>   field :value, {\n  ...>    :flags,\n  ...>    %{\n  ...>     type: :uint32_le,\n  ...>     values: [\n  ...>       { 0x00000001, :info_mouse },\n  ...>       { 0x00000002, :info_disablectrlaltdel }\n  ...>     ]\n  ...>    }\n  ...>  }\n  ...> end\n  ...>\n  ...> IntegerMaskFlags.new(value: [:info_mouse, :info_disablectrlaltdel])\n  ...> |> IntegerMaskFlags.dump_binary()\n  ...> |> IntegerMaskFlags.parse()\n  ...> |> then(fn {:ok, struct, _rest } -> struct end)\n  ...> |> IntegerMaskFlags.decode()\n  %{ value: [:info_mouse, :info_disablectrlaltdel] }\n\n\n```\n\nI found it's counterintuitively but preferred way to define flags using mask, not position.\nYou can change endianness from little to big and it will be work the same, automatically reversing all positions.\n\nBelow is list of supported masks for developers to copy and paste from. Library checks its valid mask,\n(covering only 1 bit and never overlaps) so don't worry you will not be able to mistaken it.\n\n```\n\n  def supported_masking_values(value) do\n\n    case value do\n        0x01 -> 1\n        0x02 -> 2\n        0x04 -> 3\n        0x08 -> 4\n        0x10 -> 5\n        0x20 -> 6\n        0x40 -> 7\n        0x80 -> 8\n        0x100 -> 9\n        0x200 -> 10\n        0x400 -> 11\n        0x800 -> 12\n        0x1000 -> 13\n        0x2000 -> 14\n        0x4000 -> 15\n        0x8000 -> 16\n        0x10000 -> 17\n        0x20000 -> 18\n        0x40000 -> 19\n        0x80000 -> 20\n        0x100000 -> 21\n        0x200000 -> 22\n        0x400000 -> 23\n        0x800000 -> 24\n        0x1000000 -> 25\n        0x2000000 -> 26\n        0x4000000 -> 27\n        0x8000000 -> 28\n        0x10000000 -> 29\n        0x20000000 -> 30\n        0x40000000 -> 31\n        0x80000000 -> 32\n        0x100000000 -> 33\n        0x200000000 -> 34\n        0x400000000 -> 35\n        0x800000000 -> 36\n        0x1000000000 -> 37\n        0x2000000000 -> 38\n        0x4000000000 -> 39\n        0x8000000000 -> 40\n        0x10000000000 -> 41\n        0x20000000000 -> 42\n        0x40000000000 -> 43\n        0x80000000000 -> 44\n        0x100000000000 -> 45\n        0x200000000000 -> 46\n        0x400000000000 -> 47\n        0x800000000000 -> 48\n        0x1000000000000 -> 49\n        0x2000000000000 -> 50\n        0x4000000000000 -> 51\n        0x8000000000000 -> 52\n        0x10000000000000 -> 53\n        0x20000000000000 -> 54\n        0x40000000000000 -> 55\n        0x80000000000000 -> 56\n        0x100000000000000 -> 57\n        0x200000000000000 -> 58\n        0x400000000000000 -> 59\n        0x800000000000000 -> 60\n        0x1000000000000000 -> 61\n        0x2000000000000000 -> 62\n        0x4000000000000000 -> 63\n        0x8000000000000000 -> 64\n      end\n\n  end\n\n```","ref":"flags.html#defining-struct-with-flags-using-integer-mask"},{"type":"extras","title":"Variable bit size - flags","doc":"```\n\n  iex> defmodule IntegerVariableBitSizeMaskFlags do\n  ...>   use BinStruct\n  ...>   field :value, {\n  ...>    :flags,\n  ...>    %{\n  ...>     type: :uint_le,\n  ...>     values: [\n  ...>       { 0x00000001, :info_mouse },\n  ...>       { 0x00000002, :info_disablectrlaltdel }\n  ...>     ]\n  ...>    }\n  ...>  }, bits: 24\n  ...> end\n  ...>\n  ...> IntegerVariableBitSizeMaskFlags.new(value: [:info_mouse, :info_disablectrlaltdel])\n  ...> |> IntegerVariableBitSizeMaskFlags.dump_binary()\n  ...> |> IntegerVariableBitSizeMaskFlags.parse()\n  ...> |> then(fn {:ok, struct, _rest } -> struct end)\n  ...> |> IntegerVariableBitSizeMaskFlags.decode()\n  %{ value: [:info_mouse, :info_disablectrlaltdel] }\n\n```","ref":"flags.html#variable-bit-size"},{"type":"extras","title":"float32_be","doc":"# float32_be\n\n32 bit big endian float\n\n```elixir\n\niex> defmodule Struct do\n...>   use BinStruct\n...>   field :value, :float32_be\n...> end\n...>\n...> Struct.new(value: 1.0)\n...> |> Struct.dump_binary()\n...> |> Struct.parse()\n...> |> then(fn {:ok, struct, _rest } -> struct end)\n...> |> Struct.decode()\n%{ value: 1.0 }\n\n```","ref":"float32_be.html"},{"type":"extras","title":"float32_le","doc":"# float32_le\n\n32 bit low endian float\n\n```elixir\n\n  iex> defmodule Struct do\n  ...>   use BinStruct\n  ...>   field :value, :float32_le\n  ...> end\n  ...>\n  ...> Struct.new(value: 1.0)\n  ...> |> Struct.dump_binary()\n  ...> |> Struct.parse()\n  ...> |> then(fn {:ok, struct, _rest } -> struct end)\n  ...> |> Struct.decode()\n  %{ value: 1.0 }\n\n```","ref":"float32_le.html"},{"type":"extras","title":"float64_be","doc":"# float64_be\n\n64 bit big endian float\n\n```elixir\n\n  iex> defmodule Struct do\n  ...>   use BinStruct\n  ...>   field :value, :float64_be\n  ...> end\n  ...>\n  ...> Struct.new(value: 1.0)\n  ...> |> Struct.dump_binary()\n  ...> |> Struct.parse()\n  ...> |> then(fn {:ok, struct, _rest } -> struct end)\n  ...> |> Struct.decode()\n  %{ value: 1.0 }\n\n```","ref":"float64_be.html"},{"type":"extras","title":"float64_le","doc":"# float64_le\n\n64 bit low endian float\n\n```elixir\n\n  iex> defmodule Struct do\n  ...>   use BinStruct\n  ...>   field :value, :float64_le\n  ...> end\n  ...>\n  ...> Struct.new(value: 1.0)\n  ...> |> Struct.dump_binary()\n  ...> |> Struct.parse()\n  ...> |> then(fn {:ok, struct, _rest } -> struct end)\n  ...> |> Struct.decode()\n  %{ value: 1.0 }\n\n```","ref":"float64_le.html"},{"type":"extras","title":"int16_be","doc":"# int16_be\n\n16 bit big endian signed integer\n\n```elixir\n\n  iex> defmodule Struct do\n  ...>   use BinStruct\n  ...>   field :value, :int16_be\n  ...> end\n  ...>\n  ...> Struct.new(value: -1)\n  ...> |> Struct.dump_binary()\n  ...> |> Struct.parse()\n  ...> |> then(fn {:ok, struct, _rest } -> struct end)\n  ...> |> Struct.decode()\n  %{ value: -1 }\n\n```","ref":"int16_be.html"},{"type":"extras","title":"int16_le","doc":"# int16_le\n\n16 bit low endian signed integer\n\n```elixir\n\n  iex> defmodule Struct do\n  ...>   use BinStruct\n  ...>   field :value, :int16_le\n  ...> end\n  ...>\n  ...> Struct.new(value: -1)\n  ...> |> Struct.dump_binary()\n  ...> |> Struct.parse()\n  ...> |> then(fn {:ok, struct, _rest } -> struct end)\n  ...> |> Struct.decode()\n  %{ value: -1 }\n\n```","ref":"int16_le.html"},{"type":"extras","title":"int32_be","doc":"# int32_be\n\n32 bit big endian signed integer\n\n```elixir\n\n  iex> defmodule Struct do\n  ...>   use BinStruct\n  ...>   field :value, :int32_be\n  ...> end\n  ...>\n  ...> Struct.new(value: -1)\n  ...> |> Struct.dump_binary()\n  ...> |> Struct.parse()\n  ...> |> then(fn {:ok, struct, _rest } -> struct end)\n  ...> |> Struct.decode()\n  %{ value: -1 }\n\n```","ref":"int32_be.html"},{"type":"extras","title":"int32_le","doc":"# int32_le\n\n32 bit low endian signed integer\n\n```elixir\n\n  iex> defmodule Struct do\n  ...>   use BinStruct\n  ...>   field :value, :int32_le\n  ...> end\n  ...>\n  ...> Struct.new(value: -1)\n  ...> |> Struct.dump_binary()\n  ...> |> Struct.parse()\n  ...> |> then(fn {:ok, struct, _rest } -> struct end)\n  ...> |> Struct.decode()\n  %{ value: -1 }\n\n```","ref":"int32_le.html"},{"type":"extras","title":"int64_be","doc":"# int64_be\n\n64 bit big endian signed integer\n\n```elixir\n\n  iex> defmodule Struct do\n  ...>   use BinStruct\n  ...>   field :value, :int64_be\n  ...> end\n  ...>\n  ...> Struct.new(value: -1)\n  ...> |> Struct.dump_binary()\n  ...> |> Struct.parse()\n  ...> |> then(fn {:ok, struct, _rest } -> struct end)\n  ...> |> Struct.decode()\n  %{ value: -1 }\n\n```","ref":"int64_be.html"},{"type":"extras","title":"int64_le","doc":"# int64_le\n\n64 bit low endian signed integer\n\n```elixir\n\n  iex> defmodule Struct do\n  ...>   use BinStruct\n  ...>   field :value, :int64_le\n  ...> end\n  ...>\n  ...> Struct.new(value: -1)\n  ...> |> Struct.dump_binary()\n  ...> |> Struct.parse()\n  ...> |> then(fn {:ok, struct, _rest } -> struct end)\n  ...> |> Struct.decode()\n  %{ value: -1 }\n\n```","ref":"int64_le.html"},{"type":"extras","title":"int8","doc":"# int8\n\n8 bit signed integer\n\n```elixir\n\n  iex> defmodule Struct do\n  ...>   use BinStruct\n  ...>   field :value, :int8\n  ...> end\n  ...>\n  ...> Struct.new(value: -1)\n  ...> |> Struct.dump_binary()\n  ...> |> Struct.parse()\n  ...> |> then(fn {:ok, struct, _rest } -> struct end)\n  ...> |> Struct.decode()\n  %{ value: -1 }\n\n```","ref":"int8.html"},{"type":"extras","title":"int_be","doc":"# int_be\n\nVariable size big endian integer","ref":"int_be.html"},{"type":"extras","title":"Variable bit size - int_be","doc":"```elixir\n\n  iex> defmodule Struct do\n  ...>   use BinStruct\n  ...>   field :value, :int_be, bits: 24\n  ...> end\n  ...>\n  ...> Struct.new(value: -1)\n  ...> |> Struct.dump_binary()\n  ...> |> Struct.parse()\n  ...> |> then(fn {:ok, struct, _rest } -> struct end)\n  ...> |> Struct.decode()\n  %{ value: -1 }\n\n```","ref":"int_be.html#variable-bit-size"},{"type":"extras","title":"int_le","doc":"# int_le\n\nVariable size low endian integer","ref":"int_le.html"},{"type":"extras","title":"Variable bit size - int_le","doc":"```elixir\n\n  iex> defmodule Struct do\n  ...>   use BinStruct\n  ...>   field :value, :int_le, bits: 24\n  ...> end\n  ...>\n  ...> Struct.new(value: -1)\n  ...> |> Struct.dump_binary()\n  ...> |> Struct.parse()\n  ...> |> then(fn {:ok, struct, _rest } -> struct end)\n  ...> |> Struct.decode()\n  %{ value: -1 }\n\n```","ref":"int_le.html#variable-bit-size"},{"type":"extras","title":"list_of","doc":"# list_of\n\nThere is 3 types of list_of supported","ref":"list_of.html"},{"type":"extras","title":"With compile time known bounds - list_of","doc":"BinStruct needs to know at least 2 params out of length (size in bytes), item size (in bytes) or items count\n\nItem size will be inferred if possible in case its primitive with known size or module with known_total_size_bytes\n\n```elixir\n\n  iex> defmodule CompileTimeKnownStruct do\n  ...>   use BinStruct\n  ...>   field :items, { :list_of, :uint32_be }, count: 2\n  ...> end\n  ...>\n  ...> CompileTimeKnownStruct.new(items: [ 1, 2 ])\n  ...> |> CompileTimeKnownStruct.dump_binary()\n  ...> |> CompileTimeKnownStruct.parse()\n  ...> |> then(fn {:ok, struct, _rest } -> struct end)\n  ...> |> CompileTimeKnownStruct.decode()\n  %{ items: [ 1, 2 ] }\n\n```","ref":"list_of.html#with-compile-time-known-bounds"},{"type":"extras","title":"With runtime known bounds - list_of","doc":"Same as with compile time known bounds, but any of 2 bounds could be given via registered callback\n\nOverhead of dynamic bounds comparing to known in compile time almost negligible\n\n\n```elixir\n\n  iex> defmodule RuntimeStruct do\n  ...>   use BinStruct\n  ...>\n  ...>   register_callback &count/0\n  ...>\n  ...>   field :items, { :list_of, :uint32_be }, count_by: &count/0\n  ...>\n  ...>   defp count(), do: 2\n  ...> end\n  ...>\n  ...> RuntimeStruct.new(items: [ 1, 2 ])\n  ...> |> RuntimeStruct.dump_binary()\n  ...> |> RuntimeStruct.parse()\n  ...> |> then(fn {:ok, struct, _rest } -> struct end)\n  ...> |> RuntimeStruct.decode()\n  %{ items: [ 1, 2 ] }\n\n```","ref":"list_of.html#with-runtime-known-bounds"},{"type":"extras","title":"With only one criteria known - list_of","doc":"Struct below has only one criteria (item size)\n\nIt still can be parsed, but such struct called \"not terminated\" and only parse_exact/2 function will be generated\n\nSo in case to parse this struct caller (manually called by user or nested in another BinStruct) should provide finite input.\n\n```elixir\n\n  iex> defmodule OnlyItemSizeCriteriaStruct do\n  ...>   use BinStruct\n  ...>   field :items, { :list_of, :uint32_be }\n  ...> end\n  ...>\n  ...> OnlyItemSizeCriteriaStruct.new(items: [ 1, 2 ])\n  ...> |> OnlyItemSizeCriteriaStruct.dump_binary()\n  ...> |> OnlyItemSizeCriteriaStruct.parse_exact()\n  ...> |> then(fn {:ok, struct } -> struct end)\n  ...> |> OnlyItemSizeCriteriaStruct.decode()\n  %{ items: [ 1, 2 ] }\n\n```","ref":"list_of.html#with-only-one-criteria-known"},{"type":"extras","title":"Without any bounds criteria - list_of","doc":"It's still possible to parse struct even then we are not know about its size anything, for example\n\n```elixir\n\n    iex> defmodule Item do\n    ...>   use BinStruct\n    ...>\n    ...>   register_callback &len_by/1, len: :field\n    ...>\n    ...>   field :len, :uint16_be\n    ...>   field :value, :binary, length_by: &len_by/1\n    ...>\n    ...>   defp len_by(len), do: len\n    ...> end\n    ...>\n    ...> defmodule WithoutBoundsStruct do\n    ...>   use BinStruct\n    ...>   field :items, { :list_of, Item }\n    ...> end\n    ...>\n    ...> WithoutBoundsStruct.new(items: [ Item.new(len: 1, value: \"A\"), Item.new(len: 1, value: \"B\"), Item.new(len: 1, value: \"C\") ])\n    ...> |> WithoutBoundsStruct.dump_binary()\n    ...> |> WithoutBoundsStruct.parse_exact()\n    ...> |> then(fn {:ok, struct } -> struct end)\n    ...> |> WithoutBoundsStruct.decode()\n    %{ items: [ Item.new(len: 1, value: \"A\"), Item.new(len: 1, value: \"B\"), Item.new(len: 1, value: \"C\") ] }\n\n```","ref":"list_of.html#without-any-bounds-criteria"},{"type":"extras","title":"With manual selection upon dynamic criteria - list_of","doc":"We can restrict example from \"Without any bounds criteria\" even future using take_while_by callback to any dynamic criteria\n\n```elixir\n\n    iex> defmodule TakeWhileExampleItem do\n    ...>   use BinStruct\n    ...>\n    ...>   register_callback &len_by/1, len: :field\n    ...>\n    ...>   field :len, :uint16_be\n    ...>   field :value, :binary, length_by: &len_by/1\n    ...>\n    ...>   defp len_by(len), do: len\n    ...> end\n    ...>\n    ...> defmodule WithTakeWhileByStruct do\n    ...>   use BinStruct\n    ...>\n    ...>   register_callback &take_while_by/1, items: :field\n    ...>\n    ...>   field :items, { :list_of, TakeWhileExampleItem }, take_while_by: &take_while_by/1\n    ...>\n    ...>   defp take_while_by(items) do\n    ...>\n    ...>    [ current_item | _previous_items ] = items\n    ...>    case TakeWhileExampleItem.decode(current_item) do\n    ...>      %{ value: \"C\" } -> :halt\n    ...>      _ -> :cont\n    ...>    end\n    ...>   end\n    ...> end\n    ...>\n    ...> WithTakeWhileByStruct.new(items: [ TakeWhileExampleItem.new(len: 1, value: \"A\"), TakeWhileExampleItem.new(len: 1, value: \"B\"), TakeWhileExampleItem.new(len: 1, value: \"C\") ])\n    ...> |> WithTakeWhileByStruct.dump_binary()\n    ...> |> WithTakeWhileByStruct.parse()\n    ...> |> then(fn {:ok, struct, _rest } -> struct end)\n    ...> |> WithTakeWhileByStruct.decode()\n    %{ items: [ TakeWhileExampleItem.new(len: 1, value: \"A\"), TakeWhileExampleItem.new(len: 1, value: \"B\"), TakeWhileExampleItem.new(len: 1, value: \"C\") ] }\n\n```\n\nNow struct can be parsed from infinity bytestream (parse/2 function will be available)\n\nNotice callback takes field itself to which he applied,\nwhich is normally not possible and its unique behaviour of take_while_by callback.\n\nAlso notice items are reversed, this is expected to elixir/erlang linked list implementation nature as it more performant\nto both produce and to read.\n\nType conversions are specially optimized for this callback,\ntype conversion for any item will acquire only once for each item of each type conversion requested","ref":"list_of.html#with-manual-selection-upon-dynamic-criteria"},{"type":"extras","title":"Future exploring - list_of","doc":"Most detailed behaviours can be found in test modules in BinStructTest.ListOfTests.*","ref":"list_of.html#future-exploring"},{"type":"extras","title":"Static value","doc":"# Static value\n\nStatic values are useful in case of clear declaration and validation in general\nbut super useful for dispatching variants dynamically. This behaviour will be explained more detail in variant_of docs section.\n\nYou don't need to pass static values when creating new struct and they are validated upon parsing.\nStatic values are not stored in any way but always inlined instead.","ref":"static_value.html"},{"type":"extras","title":"Inline bytes - Static value","doc":"```elixir\n\niex> defmodule StructInlineBytes do\n...>   use BinStruct\n...>   field :value, <<1, 2, 3>>\n...> end\n...>\n...> StructInlineBytes.new()\n...> |> StructInlineBytes.dump_binary()\n...> |> StructInlineBytes.parse()\n...> |> then(fn {:ok, struct, _rest } -> struct end)\n...> |> StructInlineBytes.decode()\n%{ value: <<1, 2, 3>> }\n\n```","ref":"static_value.html#inline-bytes"},{"type":"extras","title":"Optional - Static value","doc":"```elixir\n\niex> defmodule StructOptionalStaticValueNotPresent do\n...>   use BinStruct\n...>   field :value, <<1, 2, 3>>, optional: true\n...> end\n...>\n...> StructOptionalStaticValueNotPresent.new()\n...> |> StructOptionalStaticValueNotPresent.dump_binary()\n...> |> StructOptionalStaticValueNotPresent.parse_exact()\n...> |> then(fn {:ok, struct } -> struct end)\n...> |> StructOptionalStaticValueNotPresent.decode()\n%{ value: nil }\n\niex> defmodule StructOptionalStaticValuePresent do\n...>   use BinStruct\n...>   field :value, <<1, 2, 3>>, optional: true\n...> end\n...>\n...> StructOptionalStaticValuePresent.new(value: :present)\n...> |> StructOptionalStaticValuePresent.dump_binary()\n...> |> StructOptionalStaticValuePresent.parse_exact()\n...> |> then(fn {:ok, struct } -> struct end)\n...> |> StructOptionalStaticValuePresent.decode()\n%{ value: <<1, 2, 3>> }\n\n```","ref":"static_value.html#optional"},{"type":"extras","title":"Inline bitstring - Static value","doc":"```elixir\n\niex> defmodule StructInlineBitstring do\n...>   use BinStruct\n...>   field :first, <<1::1, 1::1, 1::1, 1::1>>\n...>   field :second, <<0::1, 0::1, 0::1, 0::1>>\n...> end\n...>\n...> StructInlineBitstring.new()\n...> |> StructInlineBitstring.dump_binary()\n...> |> StructInlineBitstring.parse()\n...> |> then(fn {:ok, struct, _rest } -> struct end)\n...> |> StructInlineBitstring.decode()\n%{ first: <<1::1, 1::1, 1::1, 1::1>>, second: <<0::1, 0::1, 0::1, 0::1>> }\n\n```","ref":"static_value.html#inline-bitstring"},{"type":"extras","title":"Inline String.t - Static value","doc":"```elixir\n\niex> defmodule StructInlineString do\n...>   use BinStruct\n...>   field :value, \"123\"\n...> end\n...>\n...> StructInlineString.new()\n...> |> StructInlineString.dump_binary()\n...> |> StructInlineString.parse()\n...> |> then(fn {:ok, struct, _rest } -> struct end)\n...> |> StructInlineString.decode()\n%{ value: \"123\" }\n\n```","ref":"static_value.html#inline-string-t"},{"type":"extras","title":"Evaluated - Static value","doc":"```elixir\n\niex> defmodule StructWithEvaluatedStaticValue do\n...>   use BinStruct\n...>\n...>   @my_static_constant <<1, 2, 3>>\n...>\n...>   field :v1, { :static, @my_static_constant }\n...>   field :v2, { :static, BinStruct.PrimitiveEncoder.uint8(1) }\n...>\n...> end\n...>\n...> StructWithEvaluatedStaticValue.new()\n...> |> StructWithEvaluatedStaticValue.dump_binary()\n...> |> StructWithEvaluatedStaticValue.parse()\n...> |> then(fn {:ok, struct, _rest } -> struct end)\n...> |> StructWithEvaluatedStaticValue.decode()\n%{ v1: <<1, 2, 3>>, v2: <<1>> }\n\n```","ref":"static_value.html#evaluated"},{"type":"extras","title":"uint16_be","doc":"# uint16_be\n\n16 bit big endian unsigned integer\n\n```elixir\n\n  iex> defmodule Struct do\n  ...>   use BinStruct\n  ...>   field :value, :uint16_be\n  ...> end\n  ...>\n  ...> Struct.new(value: 1)\n  ...> |> Struct.dump_binary()\n  ...> |> Struct.parse()\n  ...> |> then(fn {:ok, struct, _rest } -> struct end)\n  ...> |> Struct.decode()\n  %{ value: 1 }\n\n```","ref":"uint16_be.html"},{"type":"extras","title":"uint16_le","doc":"# uint16_le\n\n16 bit low endian unsigned integer\n\n```elixir\n\niex> defmodule Struct do\n...>   use BinStruct\n...>   field :value, :uint16_le\n...> end\n...>\n...> Struct.new(value: 1)\n...> |> Struct.dump_binary()\n...> |> Struct.parse()\n...> |> then(fn {:ok, struct, _rest } -> struct end)\n...> |> Struct.decode()\n%{ value: 1 }\n\n```","ref":"uint16_le.html"},{"type":"extras","title":"uint32_be","doc":"# uint32_be\n\n32 bit big endian unsigned integer\n\n```elixir\n\niex> defmodule Struct do\n...>   use BinStruct\n...>   field :value, :uint32_be\n...> end\n...>\n...> Struct.new(value: 1)\n...> |> Struct.dump_binary()\n...> |> Struct.parse()\n...> |> then(fn {:ok, struct, _rest } -> struct end)\n...> |> Struct.decode()\n%{ value: 1 }\n\n```","ref":"uint32_be.html"},{"type":"extras","title":"uint32_le","doc":"# uint32_le\n\n32 bit low endian unsigned integer\n\n```elixir\n\niex> defmodule Struct do\n...>   use BinStruct\n...>   field :value, :uint32_le\n...> end\n...>\n...> Struct.new(value: 1)\n...> |> Struct.dump_binary()\n...> |> Struct.parse()\n...> |> then(fn {:ok, struct, _rest } -> struct end)\n...> |> Struct.decode()\n%{ value: 1 }\n\n```","ref":"uint32_le.html"},{"type":"extras","title":"uint64_be","doc":"# uint64_be\n\n64 bit big endian unsigned integer\n\n```elixir\n\niex> defmodule Struct do\n...>   use BinStruct\n...>   field :value, :uint64_be\n...> end\n...>\n...> Struct.new(value: 1)\n...> |> Struct.dump_binary()\n...> |> Struct.parse()\n...> |> then(fn {:ok, struct, _rest } -> struct end)\n...> |> Struct.decode()\n%{ value: 1 }\n\n```","ref":"uint64_be.html"},{"type":"extras","title":"uint64_le","doc":"# uint64_le\n\n64 bit low endian unsigned integer\n\n```elixir\n\niex> defmodule Struct do\n...>   use BinStruct\n...>   field :value, :uint64_le\n...> end\n...>\n...> Struct.new(value: 1)\n...> |> Struct.dump_binary()\n...> |> Struct.parse()\n...> |> then(fn {:ok, struct, _rest } -> struct end)\n...> |> Struct.decode()\n%{ value: 1 }\n\n```","ref":"uint64_le.html"},{"type":"extras","title":"uint8","doc":"# uint8\n\n8 bit unsigned integer\n\n```elixir\n\n  iex> defmodule Struct do\n  ...>   use BinStruct\n  ...>   field :value, :uint8\n  ...> end\n  ...>\n  ...> Struct.new(value: 1)\n  ...> |> Struct.dump_binary()\n  ...> |> Struct.parse()\n  ...> |> then(fn {:ok, struct, _rest } -> struct end)\n  ...> |> Struct.decode()\n  %{ value: 1 }\n\n```","ref":"uint8.html"},{"type":"extras","title":"uint_be","doc":"# uint_be\n\nVariable size big endian unsigned integer","ref":"uint_be.html"},{"type":"extras","title":"Variable bit size - uint_be","doc":"```elixir\n\n  iex> defmodule Struct do\n  ...>   use BinStruct\n  ...>   field :value, :uint_be, bits: 24\n  ...> end\n  ...>\n  ...> Struct.new(value: 1)\n  ...> |> Struct.dump_binary()\n  ...> |> Struct.parse()\n  ...> |> then(fn {:ok, struct, _rest } -> struct end)\n  ...> |> Struct.decode()\n  %{ value: 1 }\n\n```","ref":"uint_be.html#variable-bit-size"},{"type":"extras","title":"uint_le","doc":"# uint_le\n\nVariable size low endian unsigned integer","ref":"uint_le.html"},{"type":"extras","title":"Variable bit size - uint_le","doc":"```elixir\n\niex> defmodule Struct do\n...>   use BinStruct\n...>   field :value, :uint_le, bits: 24\n...> end\n...>\n...> Struct.new(value: 1)\n...> |> Struct.dump_binary()\n...> |> Struct.parse()\n...> |> then(fn {:ok, struct, _rest } -> struct end)\n...> |> Struct.decode()\n%{ value: 1 }\n\n```","ref":"uint_le.html#variable-bit-size"},{"type":"extras","title":"variant_of","doc":"# variant_of\n\nVariantOf is way to select from two or more variants in runtime.\n\nDispatching performed using below sources:\n\n1. Performing dynamic variant dispatching using static values as source of determination validness at first place\n2. Performing dynamic variant dispatching using static values in enum declarations\n3. More complex dispatching available through dynamic callback named validate_by\n\nBinStruct will select first valid matching variant out of list. In case none is currently can be selected and\nthere is not_enough_bytes to check next it will return not_enough_bytes","ref":"variant_of.html"},{"type":"extras","title":"Static dispatch static value - variant_of","doc":"```elixir\n\niex> defmodule VariantA do\n...>   use BinStruct\n...>   field :magic_value, <<1>>\n...> end\n...>\n...> defmodule VariantB do\n...>   use BinStruct\n...>   field :magic_value, <<2>>\n...> end\n...>\n...> defmodule StructWithStaticVariants do\n...>   use BinStruct\n...>   field :one_of_other, { :variant_of, [ VariantA, VariantB ] }\n...> end\n...>\n...> StructWithStaticVariants.new(one_of_other: VariantB.new())\n...> |> StructWithStaticVariants.dump_binary()\n...> |> StructWithStaticVariants.parse()\n...> |> then(fn {:ok, struct, _rest } -> struct end)\n...> |> StructWithStaticVariants.decode()\n%{ one_of_other: VariantB.new() }\n\n```","ref":"variant_of.html#static-dispatch-static-value"},{"type":"extras","title":"Static enum - variant_of","doc":"```elixir\n\niex> defmodule KnownVariant do\n...>   use BinStruct\n...>\n...>   field :type, {\n...>     :enum,\n...>     %{\n...>         type: :binary,\n...>         values: [\n...>           \"IHDR\",\n...>           \"IEND\",\n...>         ]\n...>     }\n...>    }, length: 4\n...> end\n...>\n...> defmodule UnknownVariant do\n...>   use BinStruct\n...>   field :type, :binary, length: 4\n...> end\n...>\n...> defmodule StructWithEnumVariants do\n...>   use BinStruct\n...>   field :one_of_other, { :variant_of, [ KnownVariant, UnknownVariant ] }\n...> end\n...>\n...> StructWithEnumVariants.new(one_of_other: UnknownVariant.new(type: \"1234\"))\n...> |> StructWithEnumVariants.dump_binary()\n...> |> StructWithEnumVariants.parse()\n...> |> then(fn {:ok, struct, _rest } -> struct end)\n...> |> StructWithEnumVariants.decode()\n%{ one_of_other: UnknownVariant.new(type: \"1234\") }\n\n```","ref":"variant_of.html#static-enum"},{"type":"extras","title":"Dynamic dispatch - variant_of","doc":"```elixir\n\niex> defmodule VariantAbc do\n...>   use BinStruct\n...>\n...>   register_callback &is_abc_string/1, value: :field\n...>\n...>   field :value, :binary,\n...>      validate_by: &is_abc_string/1,\n...>      length: 3,\n...>      default: \"abc\"\n...>\n...>   defp is_abc_string(\"abc\"), do: true\n...>   defp is_abc_string(_), do: false\n...>\n...> end\n...>\n...> defmodule VariantCba do\n...>   use BinStruct\n...>\n...>   register_callback &is_cba_string/1, value: :field\n...>\n...>   field :value, :binary,\n...>      validate_by: &is_cba_string/1,\n...>      length: 3,\n...>      default: \"cba\"\n...>\n...>   defp is_cba_string(\"cba\"), do: true\n...>   defp is_cba_string(_), do: false\n...>\n...> end\n...>\n...> defmodule StructWithDynamicVariants do\n...>   use BinStruct\n...>   field :one_of_other, { :variant_of, [ VariantAbc, VariantCba ] }\n...> end\n...>\n...> StructWithDynamicVariants.new(one_of_other: VariantCba.new())\n...> |> StructWithDynamicVariants.dump_binary()\n...> |> StructWithDynamicVariants.parse()\n...> |> then(fn {:ok, struct, _rest } -> struct end)\n...> |> StructWithDynamicVariants.decode()\n%{ one_of_other: VariantCba.new() }\n\n```","ref":"variant_of.html#dynamic-dispatch"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.35.1"}}